import React, { PureComponent } from 'react';
import { message, Tooltip, Upload, Icon, Modal } from 'antd';
import styles from './index.less';
import { getSignature } from '../../services/resource';

const getFileList = (files) => {
  if (Array.isArray(files)) {
    return files.map((item, key) => {
      const urlArr = item.full_url.split('/');
      return { url: item.full_url, id: '1', uid: key, name: urlArr[urlArr.length - 1], status: 'done' };
    });
  }
  if (files && !!files.length) {
    const filesArr = files.split('/');
    return [{ uid: -1, url: files, name: filesArr[filesArr.length - 1], status: 'done' }];
  }
  return '';
}

const renderAccept = (accept) => {
  if (!accept) {
    return null;
  }
  if (['image', 'video', 'audio'].find(ext => ext === accept)) {
    return `${accept}/*`;
  }
  if (accept === 'zip') {
    return 'application/zip,application/x-zip,application/x-zip-compressed';
  }
  return `.${accept}`;
}

export default class Uploader extends PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      previewVisible: false,
      previewImage: '',
      files: getFileList(props.fileUrl),
      signature: {},
      fileName: '',
      completePath: '',
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({ files: getFileList(nextProps.fileUrl) });
  }

  genRandomFileName = (fileName, fileCode) => {
    let relativePath = (fileCode || '').replace(/-/g, '/') + '/';
    let separatorIndex = fileName.lastIndexOf('.');
    let name = fileName.substring(0, separatorIndex);
    let suffix = fileName.substring(separatorIndex + 1, fileName.length);
    let ossFileName =  ((new Date()).getTime()).toString() + Math.floor(Math.random() * 10000) + '.' + suffix;
    return `${relativePath}${ossFileName}`;
  }

  render() {
    const {
      files,
      fileName,
      signature,
      completePath,
      previewImage,
      previewVisible,
    } = this.state;
    const { accept, fileCode, onUpload, multiple = 1 } = this.props;

    const renderFiles = (fileList) => {
      const newFiles = fileList.map(file => {
        return file.response ? file.response.data.file : file;
      });
      return newFiles[0];
    }

    //upload上传参数配置
    const uploadProps = {
      accept: renderAccept(accept),
      action: signature.host,
      headers: {
        'Authorization': `OSS ${signature.accessid}:${signature.signature}`,
      },
      data: {
        policy: signature.policy,
        signature: signature.signature,
        OSSAccessKeyId: signature.accessid,
        key: `${signature.dir}${completePath}`,
        success_action_status: '200',
      },
      listType: 'picture-card',
      fileList: files,
      disabled: fileCode ? false : true,
      multiple: multiple ? true : false,
      onPreview: (file) => {
        this.setState({
          previewImage: file.url || file.thumbUrl,
          previewVisible: true,
        });
      },
      beforeUpload: (file, fileList) => {
        // 进行签名校验，失效则刷新签名
        return getSignature({ fileName: file.name })
          .then(res =>
            this.setState({
              signature: {
                ...res.data,
              },
              fileName: file.name,
              completePath: this.genRandomFileName(file.name, fileCode),
            })
          );
      },
      onChange: ({ file, fileList }) => {
        // 检查图片大小，不能超过5M
        if (file && file.size > 5 * 1024 * 1024) {
          message.error('图片大小不能超过5M!');
          return false;
        }
        // 满足要求，更新files对象
        this.setState({ files: fileList });
        // 检查上传过程中图片的状态，上传成功回传fileName, signature, file
        if (file.percent === 100 && file.status === 'done') {
          onUpload(renderFiles(fileList, 1), completePath, signature);
        } else if (file.status === 'error') {
          message.error('图片上传失败!');
        }
      },
      onRemove: (file) => {
        const fileList = this.state.files.filter(item => item.uid !== file.uid);
        onUpload(renderFiles(fileList, 0), fileName, signature);
        return true;
      },
    };

    const modalProps = {
      visible: previewVisible,
      footer: null,
      onCancel: () => this.setState({ previewVisible: false }),
    };

    const uploadButton = (
      <div>
        <Icon type="plus" />
        <div className="ant-upload-text">点击上传</div>
      </div>
    );

    return (
      <div>
        {fileCode ?
          <div>
            <Upload {...uploadProps}>
              {multiple === true ? uploadButton : (files.length < multiple && uploadButton)}
            </Upload>
            <Modal {...modalProps}>
              <img className={styles.previewImage} alt="" src={previewImage} />
            </Modal>
          </div>
        :
          <Tooltip placement="top" title="请先指定编号和名称">
            <Upload {...uploadProps}>
              {multiple === true ? uploadButton : (files.length < multiple && uploadButton)}
            </Upload>
            <Modal {...modalProps}>
              <img className={styles.previewImage} alt="" src={previewImage} />
            </Modal>
          </Tooltip>
        }
      </div>
    );
  }
}
