import fetch from 'dva/fetch';
import { message, notification } from 'antd';
import { routerRedux } from 'dva/router';
import store from '../index';

// HTTP响应状态码
const httpCodeMessage = {
  200: '服务器成功返回请求的数据',
  201: '新建或修改数据成功。',
  202: '一个请求已经进入后台排队（异步任务）',
  204: '删除数据成功。',
  400: '发出的请求有错误，服务器没有进行新建或修改数据,的操作。',
  401: '用户没有权限（令牌、用户名、密码错误）。',
  403: '用户得到授权，但是访问是被禁止的。',
  404: '发出的请求针对的是不存在的记录，服务器没有进行操作',
  406: '请求的格式不可得。',
  410: '请求的资源被永久删除，且不会再得到的。',
  422: '当创建一个对象时，发生一个验证错误。',
  500: '服务器发生错误，请检查服务器',
  502: '网关错误',
  503: '服务不可用，服务器暂时过载或维护',
  504: '网关超时',
};

// 自定义响应状态码
const customCodeMessage = {
  10004: 'Token认证失败',
  10003: 'Token失效',
  10002: '账号或密码错误',
};

/**
 * 检查HTTP响应状态码,>=200 && < 300正常
 */
function httpErrorHandler(response) {
  if (response.status >= 200 && response.status < 300) {
    return response;
  }
  const errortext = httpCodeMessage[response.status] || response.statusText;
  notification.error({
    message: `HTTP错误 ${response.status}: ${response.url}`,
    description: errortext,
  });
  const error = new Error(errortext);
  error.response = response;
  throw error;
}

/**
 * @desc 拦截接口返回的数据状态，提示错误内容
 * @return {[object]} { res: data }
 */
function apiErrorHandler(data) {
  if (!data.success) {
    const errortext = customCodeMessage[data.code] || data.message;
    // 登录密码错误
    if (data.code === 10002) {
      message.error(errortext);
    // Token失效，跳转到登录界面
    } else if (data.code === 10003 || data.code === 10004) {
      const { dispatch } = store;
      dispatch(routerRedux.push('/user/login'));
    // 其它错误打出错误代码
    } else {
      message.error(`请求错误 错误代码:${data.code} 错误信息:${errortext}`);
    }
  }
  return data;
}

/**
 * response为promise对象，转换为json
 */
function promise2Json(response) {
  return response.json();
}

/**
 * Requests a URL, returning an object or none.
 *
 * @param  {string} url       The URL we want to request
 * @param  {object} [options] The options we want to pass to "fetch"
 * @return {object}           An object containing either "data" or "err"
 */
export default function request(url, options) {
  const defaultOptions = {
    credentials: 'include', // with cookies(Post CORS requests)
  };
  const newOptions = { ...defaultOptions, ...options };
  newOptions.method = (newOptions.method || 'GET').toUpperCase();
  if (newOptions.method === 'POST' || newOptions.method === 'PUT' ||
    newOptions.method === 'PATCH' || newOptions.method === 'DELETE') {
    newOptions.headers = {
      Accept: 'application/json',
      'Content-Type': 'application/json; charset=utf-8',
      ...newOptions.headers,
    };
  } else {
    newOptions.headers = {
      Accept: 'application/json',
      ...newOptions.headers,
    };
  }

  return fetch(url, newOptions)
    .then(httpErrorHandler)
    .then(promise2Json)
    .then(apiErrorHandler)
    .catch(e => ({ e }));
}
